@@ -191,15 +191,18 @@ parse_design <- function(d_m) {
 }
 
 
-#' @title Return characteristics of a given context/d_m code (support function)
+#' @title Return characteristics of a given context/d_m code (support
+#'   function)
 #'
-#' @description Returns number of levels and model at each level.
-#' See pump_info()$Context to get a list of supported d_ms.
+#' @description Returns number of levels and model at each level. See
+#'   pump_info()$Context to get a list of supported d_ms.
 #'
 #' @param d_m string; context to parse.
+#' @param d_only TRUE/FALSE; TRUE means only look at design, ignore
+#'   model if present.
 #'
-#' @return list; list of features including number of levels,
-#' level of randomization, etc.
+#' @return list; list of features including number of levels, level of
+#'   randomization, etc.
 #'
 #' @family pump_info
 #'
@@ -208,10 +211,17 @@ parse_design <- function(d_m) {
 #' parse_d_m( supported$d_m[4] )
 #'
 #' @export
-parse_d_m <- function(d_m) {
+parse_d_m <- function(d_m, d_only=FALSE ) {
     des <- stringr::str_split(d_m, "\\.|_")[[1]]
     nums <- readr::parse_number(des)
     levels <- nums[[1]]
+    
+    if ( d_only || length(des) == 2 ) {
+        return( list( levels = levels,
+                      rand_level = nums[[2]],
+                      design = paste0( "d", levels, ".", nums[[2]] ) ) )
+    }
+    
     if ( levels == 3 ) {
         l3 <- substr( des[3], 3, 4)
         l3.p <- strsplit( l3, "" )[[1]]
@@ -707,14 +717,17 @@ validate_MTP <- function(
     chk <- MTP %in% pump_info()$Adjustment$Method
 
     if ( !all( chk ) ) {
+        val_MTPs = paste0( pump_info()$Adjustment$Method, collapse = ", " )
         if ( length( MTP ) > 1 ) {
-            msg <- sprintf( 'You have at least one invalid MTP: %s',
+            msg <- sprintf( 'You have at least one invalid MTP: %s. Valid MTPs are: %s',
                              paste( "'", MTP[!chk], "'", sep = "",
-                               collapse = ", " ) )
+                               collapse = ", " ),
+                            val_MTPs )
         } else {
-            msg <- sprintf( '"%s" is an invalid MTP.', MTP )
+            msg <- sprintf( '"%s" is an invalid MTP. Valid MTPs are: %s.', 
+                            MTP, val_MTPs )
         }
-        stop( msg )
+        stop( msg, call. = FALSE )
     }
 
     return( MTP )
@@ -749,7 +762,7 @@ validate_d_m <- function(d_m) {
                                        collapse = ", " )
                     d_m <- info$Context$d_m[[match_index[[1]]]]
                     warning(glue::glue(paste("Selecting design and model",
-                                             "{d_m} as default for design",
+                                             "{d_m} as default model for design",
                                              "from following options: {options}")),
                         call. = FALSE )
                 }
@@ -803,6 +816,11 @@ validate_inputs <- function(d_m, params.list,
         params.list$power.definition, params.list$M
     )
 
+    if ( pdef$indiv && !is.null(pdef$indiv_k) && pdef$indiv_k > params.list$M ||
+         pdef$min && pdef$min_k > params.list$M ) {
+        stop( glue::glue( "Can't calculate power '{params.list$power.definition}' for non-existant outcome (only {params.list$M} outcomes)" ), call.=FALSE )
+    }
+    
     params.list$MTP <- validate_MTP( MTP = params.list$MTP,
                                     power.call = power.call,
                                     mdes.call = mdes.call,
@@ -811,6 +829,7 @@ validate_inputs <- function(d_m, params.list,
                                     pdef = pdef,
                                     multi.MTP.ok = multi.MTP.ok )
 
+    
     #-------------------------------------------------------#
     # Westfall-Young
     #-------------------------------------------------------#
@@ -1199,6 +1218,9 @@ validate_inputs <- function(d_m, params.list,
 }
 
 
+
+
+
 #' # Not yet implemented
 #' # #' @param grid Flag of whether call is a grid call or non-grid call.
 #'
