@@ -92,6 +92,29 @@ test_that( "pumpresult for sample and mdes work", {
     expect_equal( dim(ss), c(1,4) )
     
     
+    # The print method has 'outcomes' in it
+    output <- capture.output(summary(ss))
+    expect_true(any(grepl("5 outcomes", output)))
+    
+    
+    # Check single outcome
+    set.seed( 10101033 )
+    ss <- pump_sample(   d_m = "d2.1_m2fc",
+                         nbar = 200,
+                         typesample = "J",
+                         power.definition = "min1",
+                         target.power = 0.80,
+                         M = 1,
+                         MDES = 0.05,
+                         Tbar = 0.50, alpha = 0.05,
+                         numCovar.1 = 5,
+                         R2.1 = 0.1,
+                         ICC.2 = 0.05,
+                         rho = 0, start.tnum = 100, tnum = 100,
+                         tol = 0.03 )
+    output <- capture.output(summary(ss))
+    expect_true(!any(grepl("outcomes", output)))
+    
 })
 
 
